import poser
import os
import wx
import wx.py
import wx.aui

class PathFactory:
	@classmethod
	def create(cls, pathString):
		if pathString.find("\\"):
			return OSPath(pathString)
		else:
			return PoserPath(pathString)

class PoserPath:
	def __init__(self, relativePath):   # The path which contains ":Runtime"
		self.pathString = relativePath

	def getOSPathString(self):
		return poser.ContentRootLocation() + self.pathString.replace(":", os.sep)

#	def getFileName (self):
#		return os.path.basename(self.getOSPathString())

	def asOSPath(self):
		return OSPath(self.getOSPathString())

	def asPoserPath(self):
		return self

	def isPoserPath(self):
		return True

	def isOSPath(self):
		return False

class OSPath:
	def __init__(self, absolutePath):
		self.pathString = absolutePath

	def asPoserPath(self):
		 relativePath = self.pathString.split(poser.ContentRootLocation(), 1)
		 if len(relativePath) == 1:
		 	return None
		 else:
		 	return PoserPath(relativePath[-1].replace(os.sep, ":"))

	def asOSPath(self):
		return self

	def isPoserPath(self):
		return False

	def isOSPath(self):
		return True



class RGBColor:
	def __init__(self, rgbStr="000000"):
		rgbInt = int(rgbStr[-6:], 16)
		self.b = (0x0000ff & rgbInt) / 255.0
		self.g = (0x0000ff & (rgbInt >> 8)) / 255.0
		self.r = (rgbInt >> 16) / 255.0

	def printString(self):
		return str(self.r) + ", " + str(self.g) + ", " + str(self.b)
	def getRGB(self):
		return self.r, self.g, self.b
	def setRGB(self, rgb):
		self.r=rgb[0]
		self.g=rgb[1]
		self.b=rgb[2]
	def setIntensity(self, anIntensity):
		self.r = anIntensity
		self.g = anIntensity
		self.b = anIntensity
	def setWXColor(self, aWXColor):
		(r, g, b) = aWXColor.Get()
		self.setRGB((r/255.0, g/255.0, b/255.0))
	def asRGBString(self):
		return '#%02X%02X%02X' % (int(self.r * 255), int(self.g * 255), int(self.b * 255))
	def asWXColor(self):
		return wx.Colour(int(self.r * 255), int(self.g * 255), int(self.b * 255))
	def asGrayScale(self):
		return (self.r + self.g + self.b) / 3.0
	def setByGrayScale(self, aValue):
		self.r=aValue
		self.g=aValue
		self.b=aValue

class OverallLayout:
	UPPER1 = PoserPath(":Runtime:Textures:JTrout:LDesign:Upper1.jpg").asOSPath()
	UPPER2 = PoserPath(":Runtime:Textures:JTrout:LDesign:Upper2.jpg").asOSPath()
	UPPER3 = PoserPath(":Runtime:Textures:JTrout:LDesign:Upper3.jpg").asOSPath()
	LOWER1 = PoserPath(":Runtime:Textures:JTrout:LDesign:Lower1.jpg").asOSPath()
	LOWER2 = PoserPath(":Runtime:Textures:JTrout:LDesign:Lower2.jpg").asOSPath()
	LOWER3 = PoserPath(":Runtime:Textures:JTrout:LDesign:Lower3.jpg").asOSPath()
	SIDE = PoserPath(":Runtime:Textures:JTrout:LDesign:Side.jpg").asOSPath()
	UNIFORM = PoserPath(":Runtime:Textures:JTrout:LDesign:Uniform.jpg").asOSPath()
	LayoutMap = {"Upper1":UPPER1, "Upper2":UPPER2, "Upper3":UPPER3, "Lower1":LOWER1, "Lower2":LOWER2, "Lower3":LOWER3, "Side":SIDE, "Uniform":UNIFORM}

	def __init__(self, layoutName, texture=None): # texture: OSPath
		self.thumbnailCache = {}
		self.setLayout(layoutName, texture)
		self.initEmptyThumbnail()
		self.initUnavailableThumbnail()
		self.initNotfoundThumbnail()

	def setLayout(self, layoutName, texture=None): # texture : OSPath
		self.layout = layoutName
		if layoutName == "Empty":
				self.texturePath = None
		else:
			self.texturePath = self.getTexturePathByLayout(layoutName)
			if self.texturePath == None:
				self.layout = "Custom"
				self.texturePath = texture

	def isLayoutOf(self, layoutName):
		return self.layout == layoutName

	def getTexturePathByLayout(self, layoutName):
		return(self.LayoutMap.get(layoutName, None))

	def getThumbnail(self):
		if self.layout == "Empty":
			return self.emptyThumbnail
		thumb = self.thumbnailCache.get(self.layout, None)
		if thumb:
			#print self.thumbnailCache.keys() # debug
			return thumb
		elif os.path.exists(self.texturePath.pathString):
			if wx.Image.CanRead(self.texturePath.pathString):
				img = wx.Image(self.texturePath.pathString)
				img.Rescale(125, 125)
				thumb = img.ConvertToBitmap()
				if self.layout != "Custom":
					self.thumbnailCache[self.layout] = thumb
				#print self.thumbnailCache.keys() # debug
				return thumb
			else:
				return self.unavailableThumbnail
		else:
			return self.notfoundThumbnail


	def initEmptyThumbnail(self):
		img = wx.EmptyImage(125, 125)
		img.SetRGBRect(wx.Rect(0,0,125,125), 0,0,0)
		bm = img.ConvertToBitmap()
		dc = wx.MemoryDC(bm)
		dc.SetTextForeground(wx.Color(255,255,255))
		dc.DrawText("=Empty=", 30,50)
		self.emptyThumbnail = bm

	def initUnavailableThumbnail(self):
		img = wx.EmptyImage(125, 125)
		img.SetRGBRect(wx.Rect(0,0,125,125), 0,0,0)
		bm = img.ConvertToBitmap()
		dc = wx.MemoryDC(bm)
		dc.SetTextForeground(wx.Color(255,255,255))
		dc.DrawText("=Unavailable=", 10,50)
		self.unavailableThumbnail = bm

	def initNotfoundThumbnail(self):
		img = wx.EmptyImage(125, 125)
		img.SetRGBRect(wx.Rect(0,0,125,125), 0,0,0)
		bm = img.ConvertToBitmap()
		dc = wx.MemoryDC(bm)
		dc.SetTextForeground(wx.Color(255,255,255))
		dc.DrawText("=Not Found=", 20,50)
		self.notfoundThumbnail = bm

	def getSuitableTexturePath(self):
		if (self.layout == "Custom"):
			return self.texturePath.pathString
		else:
			return ""

class IBLConfigurationHolder:

	def __init__(self, target):

		self.iblColor =  target.getIBLColor()
		self.iblIntensity =  target.getIBLIntensity()
		self.iblContrast = target.getIBLContrast()
		self.colorFront = target.getColorFront()
		self.colorBack = target.getColorBack()
		self.colorTop = target.getColorTop()
		self.colorBottom = target.getColorBottom()
		self.colorLeft = target.getColorLeft()
		self.colorRight = target.getColorRight()
		self.intensityFromFront = target.getIntensityFront()
		self.intensityFromBack = target.getIntensityBack()
		self.intensityFromTop = target.getIntensityTop()
		self.intensityFromBottom = target.getIntensityBottom()
		self.intensityFromLeft = target.getIntensityLeft()
		self.intensityFromRight = target.getIntensityRight()
		self.layout = self.getOverallLayoutByTexture(target.getBaseTexture())

	def setIBLIntensity(self, val):
		self.iblIntensity = max(0, min(float(val), 3.0))

	def setIBLContrast(self, val):
		self.iblContrast = max(1.0, min(float(val), 5.0))

	def setInternalIBLIntensity(self, val):
		self.iblInterIntensity = max(0, min(float(val), 1.0))

	def setInternalIBLIntensityNL(self, val): # For the case that MathFunction is "Max". 
		v = max(0, min(float(val), 1.0))
		self.iblInterIntensity = (2*v)/(1 + v)

	def getStringIBLIntensity(self):
		return '%1.2f'% self.iblIntensity

	def getStringIBLContrast(self):
		return '%1.1f'% self.iblContrast

	def getInternalIBLIntensityNL(self):     # For the case that MathFunction is "Max".
		return self.iblInterIntensity/(2-self.iblInterIntensity)

	def getStringInternalIBLIntensityNL(self):  # For the case that MathFunction is "Max".
		return '%1.2f'% self.getInternalIBLIntensityNL()

	def getStringInternalIBLIntensityWC(self):   # For the case that MathFunction is "Add".
		return '%1.2f'% self.getInternalIBLIntensityWC()

	def getStringInternalIBLIntensity(self):
		return '%1.2f'% self.iblInterIntensity

	def getStringIntensityFromFront(self):
		return '%1.2f'% self.intensityFromFront

	def getStringIntensityFromBack(self):
		return '%1.2f'% self.intensityFromBack

	def getStringIntensityFromTop(self):
		return '%1.2f'% self.intensityFromTop

	def getStringIntensityFromBottom(self):
		return '%1.2f'% self.intensityFromBottom

	def getStringIntensityFromLeft(self):
		return '%1.2f'% self.intensityFromLeft

	def getStringIntensityFromRight(self):
		return '%1.2f'% self.intensityFromRight

	def getOverallLayoutByTexture(self, texturePathString):
		if texturePathString == None or texturePathString == "NO_MAP" or texturePathString == "":
			return OverallLayout("Empty")
		else:
			path = PathFactory.create(texturePathString).asOSPath()
			return OverallLayout(os.path.basename(path.pathString).split('.')[0], path)

	def setLayoutName(self, layoutName):
		self.layout.setLayout(layoutName)

	def getLayoutName(self):
		return self.layout.layout

	def setLayoutToDefault(self):
		self.setLayoutName("Upper1")

	def applyTo(self, target):
		target.setColorFront(self.colorFront)
		target.setColorBack(self.colorBack)
		target.setColorTop(self.colorTop)
		target.setColorBottom(self.colorBottom)
		target.setColorLeft(self.colorLeft)
		target.setColorRight(self.colorRight)

		target.setIntensityFront(self.intensityFromFront)
		target.setIntensityBack(self.intensityFromBack)
		target.setIntensityTop(self.intensityFromTop)
		target.setIntensityBottom(self.intensityFromBottom)
		target.setIntensityLeft(self.intensityFromLeft)
		target.setIntensityRight(self.intensityFromRight)

		target.setIBLIntensity(self.iblIntensity)
		target.setIBLColor(self.iblColor)
		target.setIBLContrast(self.iblContrast)
		target.shaderTree.UpdatePreview()

		if self.layout.texturePath:
			poserPath = self.layout.texturePath.asPoserPath()
			if poserPath:
				target.setBaseTexture(poserPath.pathString)
			else:
				target.setBaseTexture(self.layout.texturePath.pathString)
		else:
			target.setBaseTexture(self.layout.texturePath)
		poser.Scene().ResolvePendingTextures()



	def getCurrentSettingString(self):
		return  self.iblColor.asRGBString() + "\n" + str(self.iblIntensity) + "\n" + str(self.iblInterIntensity) + "\n" \
+ self.colorFront.asRGBString() + "\n" + self.colorBack.asRGBString() + "\n" + self.colorTop.asRGBString() + "\n" + \
self.colorBottom.asRGBString() + "\n" + self.colorLeft.asRGBString() + "\n" + self.colorRight.asRGBString() + "\n" + \
str(self.intensityFromFront) + "\n" + str(self.intensityFromBack)  + "\n" + str(self.intensityFromTop)  + "\n" + str(self.intensityFromBottom)  + "\n" + \
str(self.intensityFromLeft) + "\n" + str(self.intensityFromRight)

	def restoreDefault(self):
		self.iblColor = RGBColor("FFFFFF")
		self.iblIntensity = 0.8
		self.iblContrast = 4.0
		#self.iblInterIntensity = 0.25
		#self.iblInterIntensity = 1.0
		self.colorFront = RGBColor("FFFFFF")
		self.colorBack = RGBColor("FFFFFF")
		self.colorTop = RGBColor("FFFFFF")
		self.colorBottom = RGBColor("FFFFFF")
		self.colorLeft = RGBColor("FFFFFF")
		self.colorRight = RGBColor("FFFFFF")
		self.intensityFromFront = 1.0
		self.intensityFromBack = 1.0
		self.intensityFromTop = 1.0
		self.intensityFromBottom = 1.0
		self.intensityFromLeft = 1.0
		self.intensityFromRight = 1.0
		self.setLayoutToDefault()


class IBLConfiguration:
	def __init__(self, aLight):
		if not self.isTargetLight(aLight):
			raise Exception, 'Not a Target light'
		self.nameMapColor = {"Side_TopBottom":"Color_Math","LeftRight_FrontBack":"Color_Math_2",\
"Top_Bottom":"Color_Math_3", "Left_Right":"Color_Math_4", "Front_Back":"Color_Math_5"}
		self.nameMapIntensity = {"Main":"Math_Functions_7", "Top":"Math_Functions","Bottom":"Math_Functions_2",\
"Left":"Math_Functions_3", "Right":"Math_Functions_4", "Front":"Math_Functions_5", "Back":"Math_Functions_6"}
		self.nameMapBlender = {"MainBlender":"Color_Math_6"}
		self.nameMapTexture = {"Base":"Image_Map_7"}
		self.IBL = aLight
		self.mat = None
		self.shaderTree = None
		self.mat = self.IBL.Material("Preview")
		self.shaderTree = self.mat.ShaderTree()

	def isTargetLight(self, aLight):
		return aLight.LightType() == poser.kLightCodeIMAGE and (aLight.InternalName().find("AltAmbient") >= 0)

	def getShaderNodeColor(self, aName):
		nodeName = self.nameMapColor[aName]
		if (nodeName):
			return self.shaderTree.NodeByInternalName(nodeName)
		else:
			return None

	def getShaderNodeIntensity(self, aName):
		nodeName = self.nameMapIntensity[aName]
		if (nodeName):
			return self.shaderTree.NodeByInternalName(nodeName)
		else:
			return None

	def getShaderNodeBlender(self, aName):
		nodeName = self.nameMapBlender[aName]
		if (nodeName):
			return self.shaderTree.NodeByInternalName(nodeName)
		else:
			return None

	def getShaderNodeTexture(self, aName):
		nodeName = self.nameMapTexture[aName]
		if (nodeName):
			return self.shaderTree.NodeByInternalName(nodeName)
		else:
			return None


	def getBaseTextureInput(self):
		node = self.getShaderNodeBlender("MainBlender")
		#node = self.getShaderNodeTexture("Base")
		#print node.Inputs() # debug
		if node:
			#return node.Input(0)
			return node.Input(1)
		else:
			return None

	def getBaseTexture(self):
		node = self.getShaderNodeTexture("Base")
		#print node.Inputs() # debug
		if node:
			return node.Input(0).Value()
		else:
			return None

	def getBaseTextureNode(self):
		return self.getShaderNodeTexture("Base")

	def getBaseTextureImageSrcInput(self):
		return self.getShaderNodeTexture("Base").Input(0)

	def setBaseTexture(self, path):
		node = self.getShaderNodeTexture("Base")
		if path == "NO_MAP":
			texturePath = None
		else:
			texturePath = path
		#print "setBaseTexture to: ", texturePath # debug
		if node:
			input = node.Input(0)
			input.SetString(texturePath)

	def setIBLIntensity(self, aValue):
		#self.shaderTree.NodeByInternalName('Light').Input(1).SetFloat(aValue)
		#self.shaderTree.NodeByInternalName('Light').InputByInternalName("Intensity").SetFloat(aValue)
		self.IBL.ParameterByCode(poser.kParmCodeKDINTENSITY).SetValue(aValue)

	def setIBLColor1(self, aRGBColor):
		(r, g, b) = aRGBColor.getRGB()
		self.IBL.ParameterByCode(poser.kParmCodeKDRED).SetValue(r)
		self.IBL.ParameterByCode(poser.kParmCodeKDGREEN).SetValue(g)
		self.IBL.ParameterByCode(poser.kParmCodeKDBLUE).SetValue(b)

	def setIBLColor2(self, aRGBColor):
		node = self.getShaderNodeBlender("MainBlender")
		node.Input(1).SetColor(aRGBColor.r, aRGBColor.g, aRGBColor.b)

	def setIBLColor(self, aRGBColor):
		node = self.getShaderNodeBlender("MainBlender")
		if node:
			self.setIBLColor2(aRGBColor)
		else:
			self.setIBLColor1(aRGBColor)

	def setIBLContrast(self, aValue):
		self.shaderTree.NodeByInternalName('Light').InputByInternalName("ImageContrast").SetFloat(aValue)

	def setColorFront(self, aRGBColor):
		b = self.getShaderNodeColor("Front_Back")
		b.Input(1).SetColor(aRGBColor.r, aRGBColor.g, aRGBColor.b)

	def setColorBack(self, aRGBColor):
		b = self.getShaderNodeColor("Front_Back")
		b.Input(2).SetColor(aRGBColor.r, aRGBColor.g, aRGBColor.b)

	def setColorTop(self, aRGBColor):
		b = self.getShaderNodeColor("Top_Bottom")
		b.Input(1).SetColor(aRGBColor.r, aRGBColor.g, aRGBColor.b)

	def setColorBottom(self, aRGBColor):
		b = self.getShaderNodeColor("Top_Bottom")
		b.Input(2).SetColor(aRGBColor.r, aRGBColor.g, aRGBColor.b)

	def setColorLeft(self, aRGBColor):
		b = self.getShaderNodeColor("Left_Right")
		b.Input(1).SetColor(aRGBColor.r, aRGBColor.g, aRGBColor.b)

	def setColorRight(self, aRGBColor):
		b = self.getShaderNodeColor("Left_Right")
		b.Input(2).SetColor(aRGBColor.r, aRGBColor.g, aRGBColor.b)

	def setIntensityFront(self, aValue):
		b = self.getShaderNodeIntensity("Front")
		b.Input(1).SetFloat(aValue)

	def setIntensityBack(self, aValue):
		b = self.getShaderNodeIntensity("Back")
		b.Input(1).SetFloat(aValue)

	def setIntensityTop(self, aValue):
		b = self.getShaderNodeIntensity("Top")
		b.Input(1).SetFloat(aValue)

	def setIntensityBottom(self, aValue):
		b = self.getShaderNodeIntensity("Bottom")
		b.Input(1).SetFloat(aValue)

	def setIntensityLeft(self, aValue):
		b = self.getShaderNodeIntensity("Left")
		b.Input(1).SetFloat(aValue)

	def setIntensityRight(self, aValue):
		b = self.getShaderNodeIntensity("Right")
		b.Input(1).SetFloat(aValue)

	def setInternalIBLIntensity(self, aValue):
		b = self.getShaderNodeIntensity("Main")
		b.Input(1).SetFloat(aValue)

	def setAttenuationFront(self, aValue):
		b = self.getShaderNodeIntensity("Front")
		b.Input(2).SetFloat(aValue)

	def setAttenuationBack(self, aValue):
		b = self.getShaderNodeIntensity("Back")
		b.Input(2).SetFloat(aValue)

	def setAttenuationTop(self, aValue):
		b = self.getShaderNodeIntensity("Top")
		b.Input(2).SetFloat(aValue)

	def setAttenuationBottom(self, aValue):
		b = self.getShaderNodeIntensity("Bottom")
		b.Input(2).SetFloat(aValue)

	def setAttenuationLeft(self, aValue):
		b = self.getShaderNodeIntensity("Left")
		b.Input(2).SetFloat(aValue)

	def setAttenuationRight(self, aValue):
		b = self.getShaderNodeIntensity("Right")
		b.Input(2).SetFloat(aValue)

	def getColorFront(self):
		b = self.getShaderNodeColor("Front_Back")
		c=RGBColor()
		c.setRGB(b.Input(1).Value())
		return c

	def getColorBack(self):
		b = self.getShaderNodeColor("Front_Back")
		c=RGBColor()
		c.setRGB(b.Input(2).Value())
		return c

	def getColorTop(self):
		b = self.getShaderNodeColor("Top_Bottom")
		c=RGBColor()
		c.setRGB(b.Input(1).Value())
		return c

	def getColorBottom(self):
		b = self.getShaderNodeColor("Top_Bottom")
		c=RGBColor()
		c.setRGB(b.Input(2).Value())
		return c

	def getColorLeft(self):
		b = self.getShaderNodeColor("Left_Right")
		c=RGBColor()
		c.setRGB(b.Input(1).Value())
		return c

	def getColorRight(self):
		b = self.getShaderNodeColor("Left_Right")
		c=RGBColor()
		c.setRGB(b.Input(2).Value())
		return c

	def getIBLIntensity(self):
		#return self.shaderTree.NodeByInternalName('Light').Input(1).Value()
		#return self.shaderTree.NodeByInternalName('Light').InputByInternalName("Intensity").Value()
		return self.IBL.ParameterByCode(poser.kParmCodeKDINTENSITY).Value()

	def getIBLColor1(self):
		rgb = (self.IBL.ParameterByCode(poser.kParmCodeKDRED).Value(), self.IBL.ParameterByCode(poser.kParmCodeKDGREEN).Value(), self.IBL.ParameterByCode(poser.kParmCodeKDBLUE).Value())
		c = RGBColor()
		c.setRGB(rgb)
		return c

	def getIBLColor2(self):
		node = self.getShaderNodeBlender("MainBlender")
		if node:
			rgb = node.Input(1).Value()
			c = RGBColor()
			c.setRGB(rgb)
			return c
		else:
			return None

	def getIBLColor(self):
		node = self.getShaderNodeBlender("MainBlender")
		if node:
			return self.getIBLColor2()
		else:
			return self.getIBLColor1()

	def getIBLContrast(self):
		return self.shaderTree.NodeByInternalName('Light').InputByInternalName("ImageContrast").Value()

	def getIntensityFront(self):
		b = self.getShaderNodeIntensity("Front")
		return b.Input(1).Value()

	def getIntensityBack(self):
		b = self.getShaderNodeIntensity("Back")
		return b.Input(1).Value()

	def getIntensityTop(self):
		b = self.getShaderNodeIntensity("Top")
		return b.Input(1).Value()

	def getIntensityBottom(self):
		b = self.getShaderNodeIntensity("Bottom")
		return b.Input(1).Value()

	def getIntensityLeft(self):
		b = self.getShaderNodeIntensity("Left")
		return b.Input(1).Value()

	def getIntensityRight(self):
		b = self.getShaderNodeIntensity("Right")
		return b.Input(1).Value()

	def getInternalIBLIntensity(self):
		b = self.getShaderNodeIntensity("Main")
		return b.Input(1).Value()

	def getAttenuationFront(self):
		b = self.getShaderNodeIntensity("Front")
		return b.Input(2).Value()

	def getAttenuationBack(self):
		b = self.getShaderNodeIntensity("Back")
		return b.Input(2).Value()

	def getAttenuationTop(self):
		b = self.getShaderNodeIntensity("Top")
		return b.Input(2).Value()

	def getAttenuationBottom(self):
		b = self.getShaderNodeIntensity("Bottom")
		return b.Input(2).Value()

	def getAttenuationLeft(self):
		b = self.getShaderNodeIntensity("Left")
		return b.Input(2).Value()

	def getAttenuationRight(self):
		b = self.getShaderNodeIntensity("Right")
		return b.Input(2).Value()


	def setAttenuationAll(self, aValue):
		self.setAttenuationFront(aValue)
		self.setAttenuationBack(aValue)
		self.setAttenuationTop(aValue)
		self.setAttenuationBottom(aValue)
		self.setAttenuationLeft(aValue)
		self.setAttenuationRight(aValue)


	def setIntensityAttenuationAndMode(self, aValue):
		b = self.getShaderNodeIntensity("Front")
		b.InputByInternalName("Math_Argument").SetFloat(3.0)  # Add = 1.0, Multiple = 3.0
		b.Input(2).SetFloat(aValue)
		b = self.getShaderNodeIntensity("Back")
		b.InputByInternalName("Math_Argument").SetFloat(3.0)
		b.Input(2).SetFloat(aValue)
		b = self.getShaderNodeIntensity("Top")
		b.InputByInternalName("Math_Argument").SetFloat(3.0)
		b.Input(2).SetFloat(aValue)
		b = self.getShaderNodeIntensity("Bottom")
		b.InputByInternalName("Math_Argument").SetFloat(3.0)
		b.Input(2).SetFloat(aValue)
		b = self.getShaderNodeIntensity("Left")
		b.InputByInternalName("Math_Argument").SetFloat(3.0)
		b.Input(2).SetFloat(aValue)
		b = self.getShaderNodeIntensity("Right")
		b.InputByInternalName("Math_Argument").SetFloat(3.0)
		b.Input(2).SetFloat(aValue)



class MainLightConfHolder: 
	def __init__(self, target):
		self.intensityDiffuse = target.getDiffuseIntensity()
		self.intensitySpecular = target.getSpecularIntensity()
		self.lightColor = target.getLightColor()
		self.lightIntensity = target.getLightIntensity()

	def setDiffuseIntensity(self, val):
		self.intensityDiffuse = max(0, min(float(val), 1.0))

	def setSpecularIntensity(self, val):
		self.intensitySpecular = max(0, min(float(val), 1.0))

	def setLightIntensity(self, val):
		self.lightIntensity = max(0, min(float(val), 3.0))

	def getStringDiffuseIntensity(self):
		return '%1.2f'% self.intensityDiffuse

	def getStringSpecularIntensity(self):
		return '%1.2f'% self.intensitySpecular

	def getStringLightIntensity(self):
		return '%1.2f'% self.lightIntensity

	def applyTo(self, target):
		target.setLightColor(self.lightColor)
		target.setLightIntensity(self.lightIntensity)
		target.setDiffuseIntensity(self.intensityDiffuse)
		target.setSpecularIntensity(self.intensitySpecular)

		target.shaderTree.UpdatePreview()

	def getCurrentSettingString(self):
		return str(self.intensityDiffuse) + "\n" + str(self.interIntensityDiffuse) + "\n" + str(self.intensitySpecular) + "\n" + self.lightColor.asRGBString() + "\n" + str(self.lightIntensity)

	def restoreDefault(self):
		self.intensityDiffuse = 1.0
		#self.interIntensityDiffuse = 0.4
		self.intensitySpecular = 1.0
		self.lightColor = RGBColor("FFFFFF")
		self.lightIntensity =  0.8


class MainLightConfiguration:
	def __init__(self, aLight):
		if not self.isTargetLight(aLight):
			raise Exception, 'Not a Target light'
		self.MainLight = aLight
		self.mat = None
		self.shaderTree = None
		self.mat = self.MainLight.Material("Preview")
		self.shaderTree = self.mat.ShaderTree()

	def isTargetLight(self, aLight):
		return aLight.LightType() == poser.kLightCodeINFINITE and (aLight.InternalName().find("MainDLight") >= 0)

	def setDiffuseColor(self, aRGBColor):
		self.shaderTree.NodeByInternalName('Light').InputByInternalName("Diffuse").SetColor(aRGBColor.r, aRGBColor.g, aRGBColor.b)

	def setDiffuseIntensity(self, aValue):
		c=RGBColor()
		c.setByGrayScale(aValue)
		self.setDiffuseColor(c)

	def setSpecularColor(self, aRGBColor):
		self.shaderTree.NodeByInternalName('Light').InputByInternalName("Specular").SetColor(aRGBColor.r, aRGBColor.g, aRGBColor.b)

	def setSpecularIntensity(self, aValue):
		c=RGBColor()
		c.setByGrayScale(aValue)
		self.setSpecularColor(c)

	def setLightColor(self, aRGBColor):
		(r, g, b) = aRGBColor.getRGB()
		self.MainLight.ParameterByCode(poser.kParmCodeKDRED).SetValue(r)
		self.MainLight.ParameterByCode(poser.kParmCodeKDGREEN).SetValue(g)
		self.MainLight.ParameterByCode(poser.kParmCodeKDBLUE).SetValue(b)

	def setLightIntensity(self, aValue):
		self.MainLight.ParameterByCode(poser.kParmCodeKDINTENSITY).SetValue(aValue)


	def getDiffuseColor(self):
		c=RGBColor()
		c.setRGB(self.shaderTree.NodeByInternalName('Light').InputByInternalName("Diffuse").Value())
		return c

	def getDiffuseIntensity(self):
		return self.getDiffuseColor().asGrayScale()

	def getSpecularColor(self):
		c=RGBColor()
		c.setRGB(self.shaderTree.NodeByInternalName('Light').InputByInternalName("Specular").Value())
		return c

	def getSpecularIntensity(self):
		return self.getSpecularColor().asGrayScale()

	def getLightIntensity(self):
		return self.MainLight.ParameterByCode(poser.kParmCodeKDINTENSITY).Value()

	def getLightColor(self):
		rgb = (self.MainLight.ParameterByCode(poser.kParmCodeKDRED).Value(), self.MainLight.ParameterByCode(poser.kParmCodeKDGREEN).Value(), self.MainLight.ParameterByCode(poser.kParmCodeKDBLUE).Value())
		c = RGBColor()
		c.setRGB(rgb)
		return c


class GIOptionHolder:
	def __init__(self, target):	
		self.GIIntensity = target.getGIIntensity()
		self.GIMaxError = target.getGIMaxError()
		self.GIOnlyRender = target.getGIOnlyRender()
		self.GINumBounces = target.getGINumBounces()
		self.GINumSamples = target.getGINumSamples()

	def applyTo(self, target):
		target.setGIIntensity(self.GIIntensity)
		target.setGIMaxError(self.GIMaxError)
		target.setGIOnlyRender(self.GIOnlyRender)
		target.setGINumBounces(self.GINumBounces)
		target.setGINumSamples(self.GINumSamples)

	def getStringGIIntensity(self):
		return '%1.2f' % self.GIIntensity

	def getGIMaxErrorString(self):
		return '%1.2f' % self.GIMaxError

	def setGIIntensity(self, val):
		self.GIIntensity = max(0.0, min(float(val), 3.0))  # TBD

	def setGIMaxError(self, val):     #   0.0 - 1.0
		self.GIMaxError = max(0.0, min(float(val), 1.0))

	def setGINumBounces(self, val):   #  1 - 12 
		self.GINumBounces = max(1, min(int(val), 12))

	def setGINumSamples(self, val):   #  64 -  2024
		self.GINumSamples = max(64, min(int(val), 2024))

	def getCurrentSettingString(self):
		return str(self.GIIntensity) + "\n" + str(self.GIMaxError)  + "\n" + str(self.GIOnlyRender) + str(self.GINumBounces) + "\n" + str(self.GINumSamples)


class GIOption:  
	def __init__(self, scene):
		self.fireFly = scene.CurrentFireFlyOptions()


	def getGIIntensity(self):
		return self.fireFly.GIIntensity()

	def getGIMaxError(self):
		return self.fireFly.GIMaxError()

	def getGIOnlyRender(self):
		return self.fireFly.GIOnlyRender()

	def getGINumBounces(self):
		return self.fireFly.GINumBounces()

	def getGINumSamples(self):
		return self.fireFly.GINumSamples()


	def setGIIntensity(self, aValue):
		self.fireFly.SetGIIntensity(aValue)

	def setGIMaxError(self, aValue):
		self.fireFly.SetGIMaxError(aValue)

	def setGIOnlyRender(self, aValue):
		self.fireFly.SetGIOnlyRender(aValue)

	def setGINumBounces(self, aValue):
		self.fireFly.SetGINumBounces(aValue)

	def setGINumSamples(self, aValue):
		self.fireFly.SetGINumSamples(aValue)


class LightingConfiguration:
	def __init__(self, aPoserScene):
		self.IBLConf = None
		self.MainLightConf = None
#		self.GIOption = GIOption(aPoserScene)
		lights = aPoserScene.Lights()
		for l in lights:
			if (self.isTargetIBL(l)):
				self.IBLConf = IBLConfiguration(l)
			if (self.isTargetMainLight(l)):
				self.MainLightConf = MainLightConfiguration(l)

	def isTargetIBL(self, aLight):
		return aLight.LightType() == poser.kLightCodeIMAGE and (aLight.InternalName().find("AltAmbient") >= 0)

	def isTargetMainLight(self, aLight):
		return aLight.LightType() == poser.kLightCodeINFINITE and (aLight.InternalName().find("MainDLight") >= 0)



class LightingConfHolder: 
	def __init__(self, target):  # class of target is LightingConfiguration
		self.ibl = IBLConfigurationHolder(target.IBLConf)
		self.mainL = MainLightConfHolder(target.MainLightConf)


	def applyTo(self, target):
		self.mainL.applyTo(target.MainLightConf)
		self.ibl.applyTo(target.IBLConf)

	def getCurrentSettingString(self):
		return str(self.GIIntensity) + "\n" +  str(self.GIOnlyRender) + "\n" + \
self.mainL.getCurrentSettingString() + "\n" +  self.ibl.getCurrentSettingString()

	def restoreDefault(self):
		self.mainL.restoreDefault()
		self.ibl.restoreDefault()


class LookPreferences:
	def __init__(self):
		self.setDefault()

	def setDefault(self):
		self.bgColor = wx.Colour(70,70,70)
		self.fgColor = wx.Colour(250,250,250)
		self.sliderSize = self.defaultSliderSize()
		self.sliderSizeWide = self.defaultSliderSizeWide()
		return self

	def setFromPoser(self):
		root = poser.WxAuiManager().GetManagedWindow()
		self.bgColor = root.GetBackgroundColour()
		self.fgColor = root.GetForegroundColour()
		self.sliderSize = self.defaultSliderSize()
		return self

	def setBgColor(self, r, g, b):
		self.bgColor = wx.Colour(r, g, b)
		return self

	def setFgColor(self, r, g, b):
		self.fgColor = wx.Colour(r, g, b)
		return self

	def setSliderSize(self, w, h):
		self.sliderSize = wx.Size(w, h)

	def applyTo(self, anObj):
		anObj.SetBackgroundColour(self.bgColor)
		anObj.SetForegroundColour(self.fgColor)
		return self

	def defaultSliderSize(self):
		return wx.Size(150, 22)

	def defaultSliderSizeWide(self):
		return wx.Size(250, 22)

class JTDialog(wx.Dialog):

	def __init__(self, aLookPreferences, aModel, title=""):
		root = self.getPoserRootWindow()
		wx.Dialog.__init__(self, root, -1, title, style=wx.DEFAULT_DIALOG_STYLE)
		self.model = aModel
		self.ColorDlg=None
		aLookPreferences.applyTo(self)
		self.init_widgets(aLookPreferences)

	# Simple layout utility

	def getNextPosV(self, aPreviousWidget, offsetX=0, offsetY=15):
		(x, y) = aPreviousWidget.GetPositionTuple()
		(w, h) = aPreviousWidget.GetSizeTuple()
		return wx.Point(x + offsetX, y + h + offsetY)

	def getNextPosH(self, aPreviousWidget, offsetX=15, offsetY=0):
		(x, y) = aPreviousWidget.GetPositionTuple()
		(w, h) = aPreviousWidget.GetSizeTuple()
		return wx.Point(x + w + offsetX, y + offsetY)

	def getNextPos(self, aPreviousWidget, offsetX=0, offsetY=0):
		(x, y) = aPreviousWidget.GetPositionTuple()
		(w, h) = aPreviousWidget.GetSizeTuple()
		return wx.Point(x + w + offsetX, y + h + offsetY)

	def getInsidePos(self, anOuterWidget, offsetX=0, offsetY=0):
		(x, y) = anOuterWidget.GetPositionTuple()
		return wx.Point(x + offsetX, y + offsetY)

	# 

	def getPoserRootWindow(self):
		#return poser.WxApp().GetTopWindow()
		return poser.WxAuiManager().GetManagedWindow()

	def init_widgets(self, aLookPreferences):
		pass    # subclass responsibility
	
	def chooseColor(self, defaultColor):
		if (self.ColorDlg == None):
			self.ColorDlg = wx.ColourDialog(self)
			colorData = self.ColorDlg.GetColourData()
			colorData.SetColour(defaultColor)
			colorData.SetChooseFull(True)
		else:
			colorData = self.ColorDlg.GetColourData()
			colorData.SetColour(defaultColor)
		if self.ColorDlg.ShowModal() == wx.ID_OK:
			#dlg.Destroy()
			return self.ColorDlg.GetColourData().GetColour()
		else:
			#dlg.Destroy()
			return None

	def Apply(self, event):
		self.EndModal(wx.ID_OK)

	def Cancel(self, event):
		self.EndModal(wx.ID_CANCEL)


class LightingConfigurationPanel(wx.Panel):
	def __init__(self, parent, model, lookPreference, id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.TAB_TRAVERSAL, name=wx.PanelNameStr):
		wx.Panel.__init__(self, parent, id, pos, size, style, name)
		self.model = model
		self.ColorDlg=None
		lookPreference.applyTo(self)
		self.init_widgets(lookPreference)

	# Simple layout utility

	def getNextPosV(self, aPreviousWidget, offsetX=0, offsetY=15):
		(x, y) = aPreviousWidget.GetPositionTuple()
		(w, h) = aPreviousWidget.GetSizeTuple()
		return wx.Point(x + offsetX, y + h + offsetY)

	def getNextPosH(self, aPreviousWidget, offsetX=15, offsetY=0):
		(x, y) = aPreviousWidget.GetPositionTuple()
		(w, h) = aPreviousWidget.GetSizeTuple()
		return wx.Point(x + w + offsetX, y + offsetY)

	def getNextPos(self, aPreviousWidget, offsetX=0, offsetY=0):
		(x, y) = aPreviousWidget.GetPositionTuple()
		(w, h) = aPreviousWidget.GetSizeTuple()
		return wx.Point(x + w + offsetX, y + h + offsetY)

	def getInsidePos(self, anOuterWidget, offsetX=0, offsetY=0):
		(x, y) = anOuterWidget.GetPositionTuple()
		return wx.Point(x + offsetX, y + offsetY)
	# 

	def getPoserRootWindow(self):
		#return poser.WxApp().GetTopWindow()
		return poser.WxAuiManager().GetManagedWindow()

	def init_widgets(self):
		pass    # subclass responsibility
	
	def chooseColor(self, defaultColor):
		if (self.ColorDlg == None):
			self.ColorDlg = wx.ColourDialog(self)
			colorData = self.ColorDlg.GetColourData()
			colorData.SetColour(defaultColor)
			colorData.SetChooseFull(True)
		else:
			colorData = self.ColorDlg.GetColourData()
			colorData.SetColour(defaultColor)
		if self.ColorDlg.ShowModal() == wx.ID_OK:
			#dlg.Destroy()
			return self.ColorDlg.GetColourData().GetColour()
		else:
			#dlg.Destroy()
			return None

	def updateIBLIntensity(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.ibl.setIBLIntensity(float(val)/100.0)
		self.valueIBLIntensity.SetValue(self.model.ibl.getStringIBLIntensity())

	def updateIBLIntensityText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		try:
			self.model.ibl.setIBLIntensity(float(val))
			obj.SetValue(self.model.ibl.getStringIBLIntensity())
			self.sldIBLIntensity.SetValue(int(self.model.ibl.iblIntensity * 100))
		except:
			obj.SetValue('0.00')
			self.sldIBLIntensity.SetValue(0)
			self.model.ibl.setIBLIntensity(0.00)

	def updateIBLContrast(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.ibl.setIBLContrast(float(val)/10.0)
		self.valueIBLContrast.SetValue(self.model.ibl.getStringIBLContrast())

	def updateIBLContrastText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		try:
			self.model.ibl.setIBLContrast(float(val))
			obj.SetValue(self.model.ibl.getStringIBLContrast())
			self.sldIBLContrast.SetValue(int(self.model.ibl.iblContrast * 10))
		except:
			obj.SetValue('0.00')
			self.sldIBLContrast.SetValue(0)
			self.model.ibl.setIBLContrast(0.00)

	def updateIBLDirectiveIntensityValueText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		try:
			val = '%1.2f'% max(0, min(float(val), 1.0))
			v = float(val)
			obj.SetValue(val)
		except:
			obj.SetValue('0.00')
			v = 0.0
		if obj.GetName()=='valueFromFront':
			self.sldFromFront.SetValue(int(v*100))
			self.model.ibl.intensityFromFront = v
		elif obj.GetName()=='valueFromBack':
			self.sldFromBack.SetValue(int(v*100))
			self.model.ibl.intensityFromBack = v
		elif obj.GetName()=='valueFromTop':
			self.sldFromTop.SetValue(int(v*100))
			self.model.ibl.intensityFromTop = v
		elif obj.GetName()=='valueFromBottom':
			self.sldFromBottom.SetValue(int(v*100))
			self.model.ibl.intensityFromBottom = v
		elif obj.GetName()=='valueFromLeft':
			self.sldFromLeft.SetValue(int(v*100))
			self.model.ibl.intensityFromLeft = v
		elif obj.GetName()=='valueFromRight':
			self.sldFromRight.SetValue(int(v*100))
			self.model.ibl.intensityFromRight = v


	def updateIBLDirectiveIntensity(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		v = float(val)/100.0
		if obj.GetName()=='sldFromFront':
			self.model.ibl.intensityFromFront = v
			self.valueFromFront.SetValue(self.model.ibl.getStringIntensityFromFront())
		elif obj.GetName()=='sldFromBack':
			self.model.ibl.intensityFromBack = v
			self.valueFromBack.SetValue(self.model.ibl.getStringIntensityFromBack())
		elif obj.GetName()=='sldFromTop':
			self.model.ibl.intensityFromTop = v
			self.valueFromTop.SetValue(self.model.ibl.getStringIntensityFromTop())
		elif obj.GetName()=='sldFromBottom':
			self.model.ibl.intensityFromBottom = v
			self.valueFromBottom.SetValue(self.model.ibl.getStringIntensityFromBottom())
		elif obj.GetName()=='sldFromLeft':
			self.model.ibl.intensityFromLeft = v
			self.valueFromLeft.SetValue(self.model.ibl.getStringIntensityFromLeft())
		elif obj.GetName()=='sldFromRight':
			self.model.ibl.intensityFromRight = v
			self.valueFromRight.SetValue(self.model.ibl.getStringIntensityFromRight())

	def setIBLDirectiveColor(self, event):
		obj = event.GetEventObject()
		rgb = RGBColor()
		rgb.setWXColor(obj.GetColour())
		if obj.GetName() == 'buttonColorFront':
			self.model.ibl.colorFront = rgb
		elif obj.GetName()=='buttonColorBack':
			self.model.ibl.colorBack = rgb
		elif obj.GetName()=='buttonColorTop':
			self.model.ibl.colorTop = rgb
		elif obj.GetName()=='buttonColorBottom':
			self.model.ibl.colorBottom = rgb
		elif obj.GetName()=='buttonColorLeft':
			self.model.ibl.colorLeft = rgb
		elif obj.GetName()=='buttonColorRight':
			self.model.ibl.colorRight = rgb


	def updateIBLColor(self, event):
		obj = event.GetEventObject()
		rgb = RGBColor()
		rgb.setWXColor(obj.GetColour())
		self.model.ibl.iblColor = rgb

	def updateGIOnly(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		if (val):
			self.model.GIOnlyRender = 1
		else:
			self.model.GIOnlyRender = 0


	def updateGIIntensityText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.setGIIntensity(float(val))
		v = self.model.GIIntensity
		obj.SetValue(self.model.getStringGIIntensity())
		self.sldGIIntensity.SetValue(int(v*100))


	def updateGIIntensity(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.setGIIntensity(float(val)/100.0)
		self.valueGIIntensity.SetValue(self.model.getStringGIIntensity())


	def updateDiffuseIntensityText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.setDiffuseGain(float(val))
		v = self.model.diffuseGain
		obj.SetValue(self.model.getStringDiffuseGain())
		self.sldDiffuseIntensity.SetValue(int(v*100))

	def updateDiffuseIntensity(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.setDiffuseGain(float(val)/100.0)
		self.valueDiffuseIntensity.SetValue(self.model.getStringDiffuseGain())

	def updateIBLRatioValueText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.setIBLRatio(float(val))
		v = self.model.iblRatio
		obj.SetValue(self.model.getStringIBLRatio())
		self.sldIBLRatio.SetValue(int(v*100))
		self.valueMLDRatio.SetValue(self.model.getStringMLDRatio())

	def updateIBLRatioValue(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.setIBLRatio(float(val)/100.0)
		self.valueIBLRatio.SetValue(self.model.getStringIBLRatio())
		self.valueMLDRatio.SetValue(self.model.getStringMLDRatio())

	def updateMLDRatioValueText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.setMLDRatio(float(val))
		obj.SetValue(self.model.getStringMLDRatio())
		v = self.model.iblRatio
		self.sldIBLRatio.SetValue(int(v*100))
		self.valueIBLRatio.SetValue(self.model.getStringIBLRatio())

	def updateMLIntensity(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.mainL.setLightIntensity(float(val)/100.0)
		self.valueMLIntensity.SetValue(self.model.mainL.getStringLightIntensity())

	def updateMLIntensityText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.mainL.setLightIntensity(float(val))
		obj.SetValue(self.model.mainL.getStringLightIntensity())
		v = self.model.mainL.lightIntensity
		self.sldMLIntensity.SetValue(int(v*100))

	def updateMLSIntensity(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.mainL.setSpecularIntensity(float(val)/100.0)
		self.valueMLSIntensity.SetValue(self.model.mainL.getStringSpecularIntensity())

	def updateMLSIntensityText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.mainL.setSpecularIntensity(float(val))
		obj.SetValue(self.model.mainL.getStringSpecularIntensity())
		v = self.model.mainL.intensitySpecular
		self.sldMLSIntensity.SetValue(int(v*100))

	def updateMLDIntensity(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.mainL.setDiffuseIntensity(float(val)/100.0)
		self.valueMLDIntensity.SetValue(self.model.mainL.getStringDiffuseIntensity())

	def updateMLDIntensityText(self, event):
		obj = event.GetEventObject()
		val = obj.GetValue()
		self.model.mainL.setDiffuseIntensity(float(val))
		obj.SetValue(self.model.mainL.getStringDiffuseIntensity())
		v = self.model.mainL.intensityDiffuse
		self.sldMLDIntensity.SetValue(int(v*100))

	def updateMLColor(self, event):
		obj = event.GetEventObject()
		rgb = RGBColor()
		rgb.setWXColor(obj.GetColour())
		self.model.mainL.lightColor = rgb

	def restoreDefault(self, event):
		pass    # subclass responsibility


	def refreshValue(self):
		pass    # subclass responsibility


class LightingConfigurationRoot(wx.Notebook):
	def __init__(self, parent, model, lookPreference, id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0, name=wx.NotebookNameStr):
		wx.Notebook.__init__(self, parent, id, pos, size, style, name)
		self.model = model
		self.basicPanel = ConfBasicPanel(self, model, lookPreference)
		self.ambPanel = AmbientDesignerPanel(self, model, lookPreference)

		self.AddPage(self.basicPanel, "Basic Configuration")
		self.AddPage(self.ambPanel, "Ambient Design")

	def getPoserRootWindow(self):
		return poser.WxAuiManager().GetManagedWindow()

	def refreshValue(self):
		self.basicPanel.refreshValue()
		self.ambPanel.refreshValue()

class ConfBasicPanel(LightingConfigurationPanel):

	def init_widgets(self, aLookPreferences):
		sldSize = aLookPreferences.sliderSize
		sldSizeWide = aLookPreferences.sliderSizeWide

	# MainLight Group

		np = wx.Point(10,15)
		self.frame3 = wx.StaticBox(id=wx.NewId(),label='Main Light Setting', name='frame3', parent=self, pos=np, size=wx.Size(280, 250), style=0)
		aLookPreferences.applyTo(self.frame3)

		# MainLightIntensity

		np = self.getInsidePos(self.frame3, offsetX=20, offsetY=30)
		self.labelMLIntensity = wx.StaticText(id=wx.NewId(),label='Main Light Intensity', name='labelMLIntensity', parent=self,pos=np, size=wx.Size(150, 21), style=0)
		np = self.getNextPosV(self.labelMLIntensity, offsetY = 5)
		self.sldMLIntensity = wx.Slider(id=wx.NewId(), maxValue=300,minValue=0, name='sldMLIntensity', parent=self, pos=np,size=sldSizeWide, style=wx.SL_HORIZONTAL, value=int(self.model.mainL.lightIntensity * 100))
		self.sldMLIntensity.SetRange(0, 300)
		self.sldMLIntensity.SetTickFreq(10, 1)
		self.sldMLIntensity.Bind(wx.EVT_SLIDER, self.updateMLIntensity)
		aLookPreferences.applyTo(self.sldMLIntensity)

		np = self.getNextPosH(self.labelMLIntensity, offsetX=10)
		self.valueMLIntensity = wx.TextCtrl(id=wx.NewId(), name='valueMLIntensity',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.mainL.getStringLightIntensity())
		self.valueMLIntensity.Bind(wx.EVT_TEXT_ENTER, self.updateMLIntensityText)
		self.valueMLIntensity.Bind(wx.EVT_KILL_FOCUS, self.updateMLIntensityText)
		aLookPreferences.applyTo(self.valueMLIntensity)

		# Mainlight Color

		np = self.getNextPosV(self.sldMLIntensity)
		self.labelMLColor = wx.StaticText(id=wx.NewId(),label='Light Color', name='labelMLColor', parent=self,pos=np, size=wx.Size(90, 13), style=0)
		np = self.getNextPosH(self.labelMLColor, offsetX = 15, offsetY=-6)
		self.buttonMLColor = wx.ColourPickerCtrl(id=wx.NewId(), col= self.model.mainL.lightColor.asWXColor(), name='buttonMLColor', parent=self, pos=np,size=wx.Size(50, 24))
		self.buttonMLColor.Bind(wx.EVT_COLOURPICKER_CHANGED, self.updateMLColor)


		# MainLightSpecular Intensity

		np = self.getNextPosV(self.labelMLColor, offsetY = 15)
		self.labelMLSIntensity = wx.StaticText(id=wx.NewId(),label='Specular Intensity', name='labelMLSIntensity', parent=self,pos=np, size=wx.Size(200, 21), style=0)
		np = self.getNextPosV(self.labelMLSIntensity, offsetY = 2)
		self.sldMLSIntensity = wx.Slider(id=wx.NewId(), maxValue=100,minValue=0, name='sldMLSIntensity', parent=self, pos=np,size=sldSize, style=wx.SL_HORIZONTAL, value=int(self.model.mainL.intensitySpecular * 100))
		self.sldMLSIntensity.SetRange(0, 100)
		self.sldMLSIntensity.SetTickFreq(10, 1)
		self.sldMLSIntensity.Bind(wx.EVT_SLIDER, self.updateMLSIntensity)
		aLookPreferences.applyTo(self.sldMLSIntensity)

		np = self.getNextPosH(self.sldMLSIntensity, offsetX=5)
		self.valueMLSIntensity = wx.TextCtrl(id=wx.NewId(), name='valueMLSIntensity',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.mainL.getStringSpecularIntensity())
		self.valueMLSIntensity.Bind(wx.EVT_TEXT_ENTER, self.updateMLSIntensityText)
		self.valueMLSIntensity.Bind(wx.EVT_KILL_FOCUS, self.updateMLSIntensityText)
		aLookPreferences.applyTo(self.valueMLSIntensity)

		# MainLightDiffuse Intensity

		np = self.getNextPosV(self.sldMLSIntensity, offsetY = 10)
		self.labelMLDIntensity = wx.StaticText(id=wx.NewId(),label='Diffuse Intensity', name='labelMLDIntensity', parent=self,pos=np, size=wx.Size(200, 21), style=0)
		np = self.getNextPosV(self.labelMLDIntensity, offsetY = 2)
		self.sldMLDIntensity = wx.Slider(id=wx.NewId(), maxValue=100,minValue=0, name='sldMLDIntensity', parent=self, pos=np,size=sldSize, style=wx.SL_HORIZONTAL, value=int(self.model.mainL.intensityDiffuse * 100))
		self.sldMLDIntensity.SetRange(0, 100)
		self.sldMLDIntensity.SetTickFreq(10, 1)
		self.sldMLDIntensity.Bind(wx.EVT_SLIDER, self.updateMLDIntensity)
		aLookPreferences.applyTo(self.sldMLDIntensity)

		np = self.getNextPosH(self.sldMLDIntensity, offsetX=5)
		self.valueMLDIntensity = wx.TextCtrl(id=wx.NewId(), name='valueMLDIntensity',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.mainL.getStringDiffuseIntensity())
		self.valueMLDIntensity.Bind(wx.EVT_TEXT_ENTER, self.updateMLDIntensityText)
		self.valueMLDIntensity.Bind(wx.EVT_KILL_FOCUS, self.updateMLDIntensityText)
		aLookPreferences.applyTo(self.valueMLDIntensity)

	# Ambient Group

		np = self.getNextPosH(self.frame3, offsetX=10)
		self.frame4 = wx.StaticBox(id=wx.NewId(),label='Ambient (IBL) Setting', name='frame4', parent=self, pos=np, size=wx.Size(280, 250), style=0)
		aLookPreferences.applyTo(self.frame4)

		# IBL Intensity

		#np = wx.Point(290, 40)
		np = self.getInsidePos(self.frame4, offsetX=20, offsetY=30)
		self.labelIBLIntensity = wx.StaticText(id=wx.NewId(),label='IBL Intensity', name='labelIBLIntensity', parent=self,pos=np, size=wx.Size(150, 21), style=0)
		
		np = self.getNextPosV(self.labelIBLIntensity, offsetY = 5)
		self.sldIBLIntensity = wx.Slider(id=wx.NewId(), maxValue=300,minValue=0, name='sldIBLIntensity', parent=self, pos=np,size=sldSizeWide, style=wx.SL_HORIZONTAL, value=int(self.model.ibl.iblIntensity * 100))
		self.sldIBLIntensity.SetRange(0, 300)
		self.sldIBLIntensity.SetTickFreq(10, 1)
		self.sldIBLIntensity.Bind(wx.EVT_SLIDER, self.updateIBLIntensity)
		aLookPreferences.applyTo(self.sldIBLIntensity)

		#np = self.getNextPosH(self.sldIBLIntensity, offsetX=5)
		np = self.getNextPosH(self.labelIBLIntensity, offsetX=10)
		self.valueIBLIntensity = wx.TextCtrl(id=wx.NewId(), name='valueIBLIntensity',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.ibl.getStringIBLIntensity())
		self.valueIBLIntensity.Bind(wx.EVT_TEXT_ENTER, self.updateIBLIntensityText)
		self.valueIBLIntensity.Bind(wx.EVT_KILL_FOCUS, self.updateIBLIntensityText)
		aLookPreferences.applyTo(self.valueIBLIntensity)

		# Ambient Color

		np = self.getNextPosV(self.sldIBLIntensity)
		self.labelIBLColor = wx.StaticText(id=wx.NewId(),label='Ambient Color', name='labelIBLColor', parent=self,pos=np, size=wx.Size(90, 13), style=0)
		np = self.getNextPosH(self.labelIBLColor, offsetX = 15, offsetY=-6)
		self.buttonIBLColor = wx.ColourPickerCtrl(id=wx.NewId(), col= self.model.ibl.iblColor.asWXColor(), name='buttonIBLColor', parent=self, pos=np,size=wx.Size(50, 24))
		self.buttonIBLColor.Bind(wx.EVT_COLOURPICKER_CHANGED, self.updateIBLColor)

		# IBL Contrast

		np = self.getNextPosV(self.labelIBLColor, offsetY=15)
		self.labelIBLContrast = wx.StaticText(id=wx.NewId(),label='IBL Contrast', name='labelIBLContrast', parent=self,pos=np, size=wx.Size(150, 21), style=0)
		
		np = self.getNextPosV(self.labelIBLContrast, offsetY = 5)
		self.sldIBLContrast = wx.Slider(id=wx.NewId(), maxValue=50,minValue=10, name='sldIBLContrast', parent=self, pos=np,size=sldSize, style=wx.SL_HORIZONTAL, value=int(self.model.ibl.iblContrast * 10))
		self.sldIBLContrast.SetRange(10, 50)
		self.sldIBLContrast.SetTickFreq(10, 1)
		self.sldIBLContrast.Bind(wx.EVT_SLIDER, self.updateIBLContrast)
		aLookPreferences.applyTo(self.sldIBLContrast)

		np = self.getNextPosH(self.sldIBLContrast, offsetX=5)
		#np = self.getNextPosH(self.labelIBLContrast, offsetX=10)
		self.valueIBLContrast = wx.TextCtrl(id=wx.NewId(), name='valueIBLContrast',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.ibl.getStringIBLContrast())
		self.valueIBLContrast.Bind(wx.EVT_TEXT_ENTER, self.updateIBLContrastText)
		self.valueIBLContrast.Bind(wx.EVT_KILL_FOCUS, self.updateIBLContrastText)
		aLookPreferences.applyTo(self.valueIBLContrast)

	def refreshValue(self):

		self.sldMLIntensity.SetValue(int(self.model.mainL.lightIntensity * 100))
		self.valueMLIntensity.SetValue(self.model.mainL.getStringLightIntensity())
		self.buttonMLColor.SetColour(self.model.mainL.lightColor.asWXColor())
		self.sldMLSIntensity.SetValue(int(self.model.mainL.intensitySpecular * 100))
		self.valueMLSIntensity.SetValue(self.model.mainL.getStringSpecularIntensity())
		self.sldMLDIntensity.SetValue(int(self.model.mainL.intensityDiffuse * 100))
		self.valueMLDIntensity.SetValue(self.model.mainL.getStringDiffuseIntensity())

		self.sldIBLIntensity.SetValue(int(self.model.ibl.iblIntensity * 100))
		self.valueIBLIntensity.SetValue(self.model.ibl.getStringIBLIntensity())
		self.buttonIBLColor.SetColour(self.model.ibl.iblColor.asWXColor())
		self.sldIBLContrast.SetValue(int(self.model.ibl.iblContrast * 10))
		self.valueIBLContrast.SetValue(self.model.ibl.getStringIBLContrast())

		self.Refresh()
        

class AmbientDesignerPanel(LightingConfigurationPanel):  

	def init_widgets(self, aLookPreferences):
		sldSize = aLookPreferences.sliderSize

	# Overall Layout

		np = wx.Point(10,16)
		self.frame1 = wx.StaticBox(id=wx.NewId(),label='Overall Layout', name='frame1', parent=self, pos=np, size=wx.Size(280, 390), style=0)
		aLookPreferences.applyTo(self.frame1)

		#np = self.getInsidePos(self.frame1, offsetX=10, offsetY=25)
		np = wx.Point(10,25)
		self.upper1 = wx.RadioButton(id=wx.NewId(), parent=self.frame1, name='Upper1', pos=np,size=wx.Size(70, 24))
		self.upper1.SetLabel("Upper1")
		if self.model.ibl.layout.isLayoutOf("Upper1"):
			self.upper1.SetValue(1)
		self.upper1.Bind(wx.EVT_RADIOBUTTON, self.updateOverallLayout)
		aLookPreferences.applyTo(self.upper1)

		np = self.getNextPosH(self.upper1, offsetX=10)
		self.upper2 = wx.RadioButton(id=wx.NewId(), parent=self.frame1, name='Upper2', pos=np,size=wx.Size(70, 24))
		self.upper2.SetLabel("Upper2")
		if self.model.ibl.layout.isLayoutOf("Upper2"):
			self.upper2.SetValue(1)
		self.upper2.Bind(wx.EVT_RADIOBUTTON, self.updateOverallLayout)
		aLookPreferences.applyTo(self.upper2)

		np = self.getNextPosH(self.upper2, offsetX=10)
		self.upper3 = wx.RadioButton(id=wx.NewId(), parent=self.frame1, name='Upper3', pos=np,size=wx.Size(70, 24))
		self.upper3.SetLabel("Upper3")
		if self.model.ibl.layout.isLayoutOf("Upper3"):
			self.upper3.SetValue(1)
		self.upper3.Bind(wx.EVT_RADIOBUTTON, self.updateOverallLayout)
		aLookPreferences.applyTo(self.upper3)

		np = self.getNextPosV(self.upper1, offsetY=10)
		self.side = wx.RadioButton(id=wx.NewId(), parent=self.frame1, name='Side', pos=np,size=wx.Size(70, 24))
		self.side.SetLabel("Side")
		if self.model.ibl.layout.isLayoutOf("Side"):
			self.side.SetValue(1)
		self.side.Bind(wx.EVT_RADIOBUTTON, self.updateOverallLayout)
		aLookPreferences.applyTo(self.side)

		np = self.getNextPosH(self.side, offsetX=10)
		self.uniform = wx.RadioButton(id=wx.NewId(), parent=self.frame1, name='Uniform', pos=np,size=wx.Size(70, 24))
		self.uniform.SetLabel("Uniform")
		if self.model.ibl.layout.isLayoutOf("Uniform"):
			self.uniform.SetValue(1)
		self.uniform.Bind(wx.EVT_RADIOBUTTON, self.updateOverallLayout)
		aLookPreferences.applyTo(self.uniform)

		np = self.getNextPosV(self.side, offsetY=10)
		self.lower1 = wx.RadioButton(id=wx.NewId(), parent=self.frame1, name='Lower1', pos=np,size=wx.Size(70, 24))
		self.lower1.SetLabel("Lower1")
		if self.model.ibl.layout.isLayoutOf("Lower1"):
			self.lower1.SetValue(1)
		self.lower1.Bind(wx.EVT_RADIOBUTTON, self.updateOverallLayout)
		aLookPreferences.applyTo(self.lower1)

		np = self.getNextPosH(self.lower1, offsetX=10)
		self.lower2 = wx.RadioButton(id=wx.NewId(), parent=self.frame1, name='Lower2', pos=np,size=wx.Size(70, 24))
		self.lower2.SetLabel("Lower2")
		if self.model.ibl.layout.isLayoutOf("Lower2"):
			self.lower2.SetValue(1)
		self.lower2.Bind(wx.EVT_RADIOBUTTON, self.updateOverallLayout)
		aLookPreferences.applyTo(self.lower2)

		np = self.getNextPosH(self.lower2, offsetX=10)
		self.lower3 = wx.RadioButton(id=wx.NewId(), parent=self.frame1, name='Lower3', pos=np,size=wx.Size(70, 24))
		self.lower3.SetLabel("Lower3")
		if self.model.ibl.layout.isLayoutOf("Lower3"):
			self.lower3.SetValue(1)
		self.lower3.Bind(wx.EVT_RADIOBUTTON, self.updateOverallLayout)
		aLookPreferences.applyTo(self.lower3)

		np = self.getNextPosV(self.lower1, offsetX = 10, offsetY=20)
		self.thumbnailFrame = wx.StaticBitmap(id=wx.NewId(), parent=self.frame1, name='texFrame', pos=np, size=wx.Size(125,125))
		self.thumbnailFrame.SetBitmap(self.model.ibl.layout.getThumbnail())
		aLookPreferences.applyTo(self.thumbnailFrame)

        # Image Picker BUTTON
		np = self.getNextPosV(self.thumbnailFrame, offsetX = 10, offsetY=10)
		self.textureButton = wx.Button(id=wx.NewId(), parent= self.frame1, label='Custom Layout',name='textureButton', pos=np,size=wx.Size(120, 23), style=0)
		self.textureButton.Bind(wx.EVT_BUTTON, self.selectCustomLayout)
		aLookPreferences.applyTo(self.textureButton)

		np = self.getNextPosV(self.textureButton, offsetX = -20, offsetY=10)
#		self.texturePathLabel = wx.StaticText(id=wx.NewId(),name='texturePathLabel', parent=self.frame1,pos=np, size=wx.Size(230, 40), style=0)
#		self.texturePathLabel.SetLabel(self.model.ibl.layout.getSuitableTexturePath())
#		self.texturePathLabel.Wrap(150)
		self.texturePathLabel = wx.TextCtrl(id=wx.NewId(),name='texturePathLabel', parent=self.frame1,pos=np, size=wx.Size(260, 70), style=wx.TE_READONLY|wx.TE_MULTILINE)
		#self.texturePathLabel.SetEditable(False)
		aLookPreferences.applyTo(self.texturePathLabel)
		self.texturePathLabel.SetValue(self.model.ibl.layout.getSuitableTexturePath())


	# adjustment of each direction

		np = self.getNextPosH(self.frame1, offsetX=15)
		self.frame2 = wx.StaticBox(id=wx.NewId(),label='Adjustment of each direction.', name='frame2', parent=self, pos=np, size=wx.Size(270, 390), style=0)
		aLookPreferences.applyTo(self.frame2)

		# Front Intensity

		np = self.getInsidePos(self.frame2, offsetX=20, offsetY=25)
		self.labelFromFront = wx.StaticText(id=wx.NewId(),label='From Front', name='labelFromFront', parent=self,pos=np, size=wx.Size(100, 21), style=0)
		
		np = self.getNextPosV(self.labelFromFront, offsetY = 2)
		self.sldFromFront = wx.Slider(id=wx.NewId(), maxValue=100,minValue=0, name='sldFromFront', parent=self, pos=np,size=sldSize, style=wx.SL_HORIZONTAL, value=int(self.model.ibl.intensityFromFront * 100))
		self.sldFromFront.SetRange(0, 100)
		self.sldFromFront.SetTickFreq(10, 1)
		self.sldFromFront.Bind(wx.EVT_SLIDER, self.updateIBLDirectiveIntensity)
		aLookPreferences.applyTo(self.sldFromFront)

		np = self.getNextPosH(self.sldFromFront, offsetX=5)
		self.valueFromFront = wx.TextCtrl(id=wx.NewId(), name='valueFromFront',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.ibl.getStringIntensityFromFront())
		self.valueFromFront.Bind(wx.EVT_TEXT_ENTER, self.updateIBLDirectiveIntensityValueText)
		self.valueFromFront.Bind(wx.EVT_KILL_FOCUS, self.updateIBLDirectiveIntensityValueText)
		aLookPreferences.applyTo(self.valueFromFront)

		# Front Color
		np = self.getNextPosH(self.valueFromFront, offsetX=5)
		self.buttonColorFront = wx.ColourPickerCtrl(id=wx.NewId(), col= self.model.ibl.colorFront.asWXColor(), name='buttonColorFront', parent=self, pos=np,size=wx.Size(35, 24))
		self.buttonColorFront.Bind(wx.EVT_COLOURPICKER_CHANGED, self.setIBLDirectiveColor)

		# Back Intensity

		np = self.getNextPosV(self.sldFromFront, offsetY=10)
		self.labelFromBack = wx.StaticText(id=wx.NewId(),label='From Back', name='labelFromBack', parent=self,pos=np, size=wx.Size(100, 21), style=0)
		
		np = self.getNextPosV(self.labelFromBack, offsetY = 2)
		self.sldFromBack = wx.Slider(id=wx.NewId(), maxValue=100,minValue=0, name='sldFromBack', parent=self, pos=np,size=sldSize, style=wx.SL_HORIZONTAL, value=int(self.model.ibl.intensityFromBack * 100))
		self.sldFromBack.SetRange(0, 100)
		self.sldFromBack.SetTickFreq(10, 1)
		self.sldFromBack.Bind(wx.EVT_SLIDER, self.updateIBLDirectiveIntensity)
		aLookPreferences.applyTo(self.sldFromBack)

		np = self.getNextPosH(self.sldFromBack, offsetX=5)
		self.valueFromBack = wx.TextCtrl(id=wx.NewId(), name='valueFromBack',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.ibl.getStringIntensityFromBack())
		self.valueFromBack.Bind(wx.EVT_TEXT_ENTER, self.updateIBLDirectiveIntensityValueText)
		self.valueFromBack.Bind(wx.EVT_KILL_FOCUS, self.updateIBLDirectiveIntensityValueText)
		aLookPreferences.applyTo(self.valueFromBack)

		# Back Color
		np = self.getNextPosH(self.valueFromBack, offsetX=5)
		self.buttonColorBack = wx.ColourPickerCtrl(id=wx.NewId(), col= self.model.ibl.colorBack.asWXColor(), name='buttonColorBack', parent=self, pos=np,size=wx.Size(35, 24))
		self.buttonColorBack.Bind(wx.EVT_COLOURPICKER_CHANGED, self.setIBLDirectiveColor)

		# Top Intensity

		np = self.getNextPosV(self.sldFromBack, offsetY=10)
		self.labelFromTop = wx.StaticText(id=wx.NewId(),label='From Top', name='labelFromTop', parent=self,pos=np, size=wx.Size(100, 21), style=0)
		
		np = self.getNextPosV(self.labelFromTop, offsetY = 2)
		self.sldFromTop = wx.Slider(id=wx.NewId(), maxValue=100,minValue=0, name='sldFromTop', parent=self, pos=np,size=sldSize, style=wx.SL_HORIZONTAL, value=int(self.model.ibl.intensityFromTop * 100))
		self.sldFromTop.SetRange(0, 100)
		self.sldFromTop.SetTickFreq(10, 1)
		self.sldFromTop.Bind(wx.EVT_SLIDER, self.updateIBLDirectiveIntensity)
		aLookPreferences.applyTo(self.sldFromTop)

		np = self.getNextPosH(self.sldFromTop, offsetX=5)
		self.valueFromTop = wx.TextCtrl(id=wx.NewId(), name='valueFromTop',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.ibl.getStringIntensityFromTop())
		self.valueFromTop.Bind(wx.EVT_TEXT_ENTER, self.updateIBLDirectiveIntensityValueText)
		self.valueFromTop.Bind(wx.EVT_KILL_FOCUS, self.updateIBLDirectiveIntensityValueText)
		aLookPreferences.applyTo(self.valueFromTop)

		# Top Color
		np = self.getNextPosH(self.valueFromTop, offsetX=5)
		self.buttonColorTop = wx.ColourPickerCtrl(id=wx.NewId(), col= self.model.ibl.colorTop.asWXColor(), name='buttonColorTop', parent=self, pos=np,size=wx.Size(35, 24))
		self.buttonColorTop.Bind(wx.EVT_COLOURPICKER_CHANGED, self.setIBLDirectiveColor)
		
		# Bottom Intensity

		np = self.getNextPosV(self.sldFromTop, offsetY=10)
		self.labelFromBottom = wx.StaticText(id=wx.NewId(),label='From Bottom', name='labelFromBottom', parent=self,pos=np, size=wx.Size(100, 21), style=0)
		
		np = self.getNextPosV(self.labelFromBottom, offsetY = 2)
		self.sldFromBottom = wx.Slider(id=wx.NewId(), maxValue=100,minValue=0, name='sldFromBottom', parent=self, pos=np,size=sldSize, style=wx.SL_HORIZONTAL, value=int(self.model.ibl.intensityFromBottom * 100))
		self.sldFromBottom.SetRange(0, 100)
		self.sldFromBottom.SetTickFreq(10, 1)
		self.sldFromBottom.Bind(wx.EVT_SLIDER, self.updateIBLDirectiveIntensity)
		aLookPreferences.applyTo(self.sldFromBottom)

		np = self.getNextPosH(self.sldFromBottom, offsetX=5)
		self.valueFromBottom = wx.TextCtrl(id=wx.NewId(), name='valueFromBottom',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.ibl.getStringIntensityFromBottom())
		self.valueFromBottom.Bind(wx.EVT_TEXT_ENTER, self.updateIBLDirectiveIntensityValueText)
		self.valueFromBottom.Bind(wx.EVT_KILL_FOCUS, self.updateIBLDirectiveIntensityValueText)
		aLookPreferences.applyTo(self.valueFromBottom)
		
		# Bottom Color
		np = self.getNextPosH(self.valueFromBottom, offsetX=5)
		self.buttonColorBottom = wx.ColourPickerCtrl(id=wx.NewId(), col= self.model.ibl.colorBottom.asWXColor(), name='buttonColorBottom', parent=self, pos=np,size=wx.Size(35, 24))
		self.buttonColorBottom.Bind(wx.EVT_COLOURPICKER_CHANGED, self.setIBLDirectiveColor)

		# Left Intensity

		np = self.getNextPosV(self.sldFromBottom, offsetY=10)
		self.labelFromLeft = wx.StaticText(id=wx.NewId(),label='From Left', name='labelFromLeft', parent=self,pos=np, size=wx.Size(100, 21), style=0)
		
		np = self.getNextPosV(self.labelFromLeft, offsetY = 2)
		self.sldFromLeft = wx.Slider(id=wx.NewId(), maxValue=100,minValue=0, name='sldFromLeft', parent=self, pos=np,size=sldSize, style=wx.SL_HORIZONTAL, value=int(self.model.ibl.intensityFromLeft * 100))
		self.sldFromLeft.SetRange(0, 100)
		self.sldFromLeft.SetTickFreq(10, 1)
		self.sldFromLeft.Bind(wx.EVT_SLIDER, self.updateIBLDirectiveIntensity)
		aLookPreferences.applyTo(self.sldFromLeft)

		np = self.getNextPosH(self.sldFromLeft, offsetX=5)
		self.valueFromLeft = wx.TextCtrl(id=wx.NewId(), name='valueFromLeft',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.ibl.getStringIntensityFromLeft())
		self.valueFromLeft.Bind(wx.EVT_TEXT_ENTER, self.updateIBLDirectiveIntensityValueText)
		self.valueFromLeft.Bind(wx.EVT_KILL_FOCUS, self.updateIBLDirectiveIntensityValueText)
		aLookPreferences.applyTo(self.valueFromLeft)

		# Left Color
		np = self.getNextPosH(self.valueFromLeft, offsetX=5)
		self.buttonColorLeft = wx.ColourPickerCtrl(id=wx.NewId(), col= self.model.ibl.colorLeft.asWXColor(), name='buttonColorLeft', parent=self, pos=np,size=wx.Size(35, 24))
		self.buttonColorLeft.Bind(wx.EVT_COLOURPICKER_CHANGED, self.setIBLDirectiveColor)

		# Right Intensity

		np = self.getNextPosV(self.sldFromLeft, offsetY=10)
		self.labelFromRight = wx.StaticText(id=wx.NewId(),label='From Right', name='labelFromRight', parent=self,pos=np, size=wx.Size(100, 21), style=0)
		
		np = self.getNextPosV(self.labelFromRight, offsetY = 2)
		self.sldFromRight = wx.Slider(id=wx.NewId(), maxValue=100,minValue=0, name='sldFromRight', parent=self, pos=np,size=sldSize, style=wx.SL_HORIZONTAL, value=int(self.model.ibl.intensityFromRight * 100))
		self.sldFromRight.SetRange(0, 100)
		self.sldFromRight.SetTickFreq(10, 1)
		self.sldFromRight.Bind(wx.EVT_SLIDER, self.updateIBLDirectiveIntensity)
		aLookPreferences.applyTo(self.sldFromRight)

		np = self.getNextPosH(self.sldFromRight, offsetX=5)
		self.valueFromRight = wx.TextCtrl(id=wx.NewId(), name='valueFromRight',parent=self, pos=np, size=wx.Size(50, 21), style=wx.TE_PROCESS_ENTER,value=self.model.ibl.getStringIntensityFromRight())
		self.valueFromRight.Bind(wx.EVT_TEXT_ENTER, self.updateIBLDirectiveIntensityValueText)
		self.valueFromRight.Bind(wx.EVT_KILL_FOCUS, self.updateIBLDirectiveIntensityValueText)
		aLookPreferences.applyTo(self.valueFromRight)

		# Right Color
		np = self.getNextPosH(self.valueFromRight, offsetX=5)
		self.buttonColorRight = wx.ColourPickerCtrl(id=wx.NewId(), col= self.model.ibl.colorRight.asWXColor(), name='buttonColorRight', parent=self, pos=np,size=wx.Size(35, 24))
		self.buttonColorRight.Bind(wx.EVT_COLOURPICKER_CHANGED, self.setIBLDirectiveColor)


	def updateOverallLayout(self, event):
		obj = event.GetEventObject()
		#v = obj.GetValue()
		#self.model.ibl.layout.setLayout(obj.GetName())
		self.model.ibl.setLayoutName(obj.GetName())
		self.thumbnailFrame.SetBitmap(self.model.ibl.layout.getThumbnail())
		self.texturePathLabel.SetValue(self.model.ibl.layout.getSuitableTexturePath())


	def selectCustomLayout(self, event):
		result = poser.DialogSimple.PickImage()
		if result != None:
			if result == "":
				self.model.ibl.setLayoutToDefault()  # Restore default
				self.thumbnailFrame.SetBitmap(self.model.ibl.layout.getThumbnail())
			else:
				self.model.ibl.layout.setLayout("Custom", OSPath(result))
				self.thumbnailFrame.SetBitmap(self.model.ibl.layout.getThumbnail())
			self.refreshOverallLayoutButtons()
			self.texturePathLabel.SetValue(self.model.ibl.layout.getSuitableTexturePath())

	def refreshOverallLayoutButtons(self):
		if self.model.ibl.layout.isLayoutOf("Upper1"):
			self.upper1.SetValue(1)
		else:
			self.upper1.SetValue(0)
		if self.model.ibl.layout.isLayoutOf("Upper2"):
			self.upper2.SetValue(1)
		else:
			self.upper2.SetValue(0)
		if self.model.ibl.layout.isLayoutOf("Upper3"):
			self.upper3.SetValue(1)
		else:
			self.upper3.SetValue(0)
		if self.model.ibl.layout.isLayoutOf("Lower1"):
			self.lower1.SetValue(1)
		else:
			self.lower1.SetValue(0)
		if self.model.ibl.layout.isLayoutOf("Lower2"):
			self.lower2.SetValue(1)
		else:
			self.lower2.SetValue(0)
		if self.model.ibl.layout.isLayoutOf("Lower3"):
			self.lower3.SetValue(1)
		else:
			self.lower3.SetValue(0)
		if self.model.ibl.layout.isLayoutOf("Side"):
			self.side.SetValue(1)
		else:
			self.side.SetValue(0)
		if self.model.ibl.layout.isLayoutOf("Uniform"):
			self.uniform.SetValue(1)
		else:
			self.uniform.SetValue(0)

	def refreshValue(self):

		self.sldFromFront.SetValue(int(self.model.ibl.intensityFromFront * 100))
		self.valueFromFront.SetValue(self.model.ibl.getStringIntensityFromFront())
		self.buttonColorFront.SetColour(self.model.ibl.colorFront.asWXColor())
		self.sldFromBack.SetValue(int(self.model.ibl.intensityFromBack * 100))
		self.valueFromBack.SetValue(self.model.ibl.getStringIntensityFromBack())
		self.buttonColorBack.SetColour(self.model.ibl.colorBack.asWXColor())
		self.sldFromTop.SetValue(int(self.model.ibl.intensityFromTop * 100))
		self.valueFromTop.SetValue(self.model.ibl.getStringIntensityFromTop())
		self.buttonColorTop.SetColour(self.model.ibl.colorTop.asWXColor())
		self.sldFromBottom.SetValue(int(self.model.ibl.intensityFromBottom * 100))
		self.valueFromBottom.SetValue(self.model.ibl.getStringIntensityFromBottom())
		self.buttonColorBottom.SetColour(self.model.ibl.colorBottom.asWXColor())
		self.sldFromLeft.SetValue(int(self.model.ibl.intensityFromLeft * 100))
		self.valueFromLeft.SetValue(self.model.ibl.getStringIntensityFromLeft())
		self.buttonColorLeft.SetColour(self.model.ibl.colorLeft.asWXColor())
		self.sldFromRight.SetValue(int(self.model.ibl.intensityFromRight * 100))
		self.valueFromRight.SetValue(self.model.ibl.getStringIntensityFromRight())
		self.buttonColorRight.SetColour(self.model.ibl.colorRight.asWXColor())
		self.refreshOverallLayoutButtons()
		self.thumbnailFrame.SetBitmap(self.model.ibl.layout.getThumbnail())
		self.texturePathLabel.SetValue(self.model.ibl.layout.getSuitableTexturePath())

		self.Refresh()


class LightingDesignerDialog(JTDialog):  

	def init_widgets(self, aLookPreferences):
		self.SetClientSize(wx.Size(610, 550))
		sldSize = aLookPreferences.sliderSize

		np = wx.Point(10,10)

		self.mainPanel = LightingConfigurationRoot(self, self.model, aLookPreferences, name= "LightingConfigurationRoot", pos = np, size=wx.Size(600,450))


        # Default BUTTON
		np = self.getNextPosV(self.mainPanel, offsetX=20, offsetY=15)
		self.defaultButton = wx.Button(id=wx.NewId(), label='Restore Default',name='defaultButton', parent=self, pos=np,size=wx.Size(120, 23), style=0)
		self.defaultButton.Bind(wx.EVT_BUTTON, self.restoreDefault)
		aLookPreferences.applyTo(self.defaultButton)

		# Cancel BUTTON
		np = self.getNextPosV(self.defaultButton, offsetY=15)
		self.cancelButton = wx.Button(id=wx.NewId(), label='Cancel',name='cancelButton', parent=self, pos=np,size=wx.Size(100, 23), style=0)
		self.cancelButton.Bind(wx.EVT_BUTTON, self.Cancel)
		aLookPreferences.applyTo(self.cancelButton)

		# Apply BUTTON
		np = self.getNextPosH(self.cancelButton, offsetX=20)
		self.applyButton = wx.Button(id=wx.NewId(), label='Apply',name='applyButton', parent=self, pos=np,size=wx.Size(100, 23), style=0)
		self.applyButton.Bind(wx.EVT_BUTTON, self.Apply)
		aLookPreferences.applyTo(self.applyButton)
		
		self.cancelButton.SetFocus()

	def restoreDefault(self, event):
		self.model.restoreDefault()
		self.refreshValue()

	def refreshValue(self):
		self.mainPanel.refreshValue()
		self.Refresh()

#--------------------------------------------------------------------------------------


def main():
	scene = poser.Scene()
	amb = LightingConfiguration(scene)
	if (amb.IBLConf and amb.MainLightConf):
		#aLP=LookPreferences().setDefault()
		aLP=LookPreferences().setFromPoser()
		model = LightingConfHolder(amb)
		#print model.getCurrentSettingString()  #debug
		dlg = LightingDesignerDialog(aLP, model, title= "Lighting Configuration")
		dlg.CenterOnScreen()
		if(dlg.ShowModal() == wx.ID_OK):
			model.applyTo(amb)
			scene.Draw()
		dlg.Destroy()
	else:
		poser.DialogSimple.MessageBox("Proper light was not found !!. \n\nPlease load \"BaseLight\" first.")

main()
del main

del LightingDesignerDialog
del JTDialog

del AmbientDesignerPanel
del ConfBasicPanel
del LightingConfigurationRoot
del LightingConfigurationPanel

del LightingConfiguration
del LightingConfHolder
del GIOptionHolder
del GIOption
del IBLConfigurationHolder
del IBLConfiguration
del MainLightConfiguration
del MainLightConfHolder

del LookPreferences
del OverallLayout
del RGBColor
del PoserPath
del OSPath
del PathFactory
